@@ -2,11 +2,12 @@
 
 # Copyright 2009 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
-from tests import TestCase
+import tests
 
 from zim.parsing import *
+from zim.parser import *
 
-class TestParsing(TestCase):
+class TestParsing(tests.TestCase):
 
 	def testSplitWords(self):
 		'''Test parsing quoted strings'''
@@ -130,3 +131,127 @@ class TestParsing(TestCase):
 		):
 			#~ print '>>', href
 			self.assertEqual(link_type(href), type)
+
+
+class TestSimpleTreeBuilder(tests.TestCase):
+
+	def runTest(self):
+		E = SimpleTreeElement
+
+		builder = SimpleTreeBuilder()
+
+		builder.start('root', {})
+		builder.text('foo')
+		builder.text('bar')
+		builder.append('dus', {}, 'ja')
+		builder.text('foo')
+		builder.text('bar')
+		builder.append('br', {})
+		builder.text('foo')
+		builder.text('bar')
+		builder.end('root')
+
+		root = builder.get_root()
+		self.assertEqual(root, [
+			E('root', {}, [
+					'foo', 'bar',
+					E('dus', {}, ['ja']),
+					'foo', 'bar',
+					E('br', {}, []),
+					'foo', 'bar',
+				]
+			)
+		])
+
+
+		realbuilder = SimpleTreeBuilder()
+		builder = BuilderTextBuffer(realbuilder)
+
+		builder.start('root', {})
+		builder.text('foo')
+		builder.text('bar')
+		builder.append('dus', {}, 'ja')
+		builder.text('foo')
+		builder.text('bar')
+		builder.append('br', {})
+		builder.text('foo')
+		builder.text('bar')
+		builder.end('root')
+
+		root = realbuilder.get_root()
+		self.assertEqual(root, [
+			E('root', {}, [
+					'foobar',
+					E('dus', {}, ['ja']),
+					'foobar',
+					E('br', {}, []),
+					'foobar',
+				]
+			)
+		])
+
+
+
+class TestBuilderTextBuffer(tests.TestCase):
+
+	def runTest(self):
+		builder = SimpleTreeBuilder()
+		buffer = BuilderTextBuffer(builder)
+
+		buffer.start('FOO')
+		buffer.text('aaa\n')
+		buffer.text('bbb\n')
+		buffer.text('ccc\n')
+		self.assertEqual(buffer.get_text(), 'aaa\nbbb\nccc\n')
+
+		buffer.append('BAR')
+		self.assertEqual(buffer.get_text(), '')
+
+		buffer.text('qqq\n')
+		self.assertEqual(buffer.get_text(), 'qqq\n')
+		buffer.clear_text()
+
+		buffer.text('qqq\n')
+		self.assertEqual(buffer.get_text(), 'qqq\n')
+		buffer.set_text('ddd\n')
+		self.assertEqual(buffer.get_text(), 'ddd\n')
+
+		buffer.text('')
+		buffer.text('eee')
+		buffer.end('FOO')
+
+		E = SimpleTreeElement
+		self.assertEqual(builder.get_root(), [
+			E('FOO', None, [
+				u'aaa\nbbb\nccc\n',
+				E('BAR', None, []),
+				u'ddd\neee',
+			])
+		])
+
+
+
+class TestParser(tests.TestCase):
+
+	def testFunctions(self):
+		# Helper functions
+		for input, wanted in (
+			('foo', 'foo\n'),
+			('foo\nbar', 'foo\nbar\n'),
+			('    foo\n\t     bar', '\tfoo\n\t\t bar\n'),
+		):
+			output = prepare_text(input)
+			self.assertEqual(output, wanted)
+
+		text = 'foo\nbar\nbaz\n'
+		for offset, wanted in (
+			(0, (1, 0)),
+			(3, (1, 3)),
+			(4, (2, 0)),
+			(8, (3, 0)),
+			(9, (3, 1)),
+		):
+			line = get_line_count(text, offset)
+			self.assertEqual(line, wanted)
+
+	## TODO -- Parser test cases ##
