@@ -2,15 +2,22 @@
 
 # Copyright 2009 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
+from __future__ import with_statement
+
+
 import tests
 
+from tests.gui import setupGtkInterface
+
 import os
 import tempfile
+import gtk
 
 from zim.fs import File, Dir
-from zim.plugins.versioncontrol import VersionControlPlugin, NoChangesError
+from zim.applications import Application
+from zim.notebook import Path
 
-from zim.plugins.versioncontrol import VCS
+from zim.plugins.versioncontrol import *
 
 import zim.plugins.versioncontrol.bzr
 import zim.plugins.versioncontrol.hg
@@ -21,10 +28,11 @@ import zim.plugins.versioncontrol.git
 # because sources are probably under change control already - want to
 # avoid mixing up the files
 def get_tmp_dir(name):
-	testtmp = os.environ['TMP']
-	del os.environ['TMP']
-	dir = Dir(tempfile.gettempdir())
-	os.environ['TMP'] = testtmp
+	if 'REAL_TMP' in os.environ: # Set in tests/__init__.py
+		dir = Dir(os.environ['REAL_TMP'])
+	else:
+		dir = Dir(tempfile.gettempdir())
+	#~ print "TMPDIR:", dir
 
 	dir = dir.subdir('test_versioncontrol').subdir(name)
 	if dir.exists():
@@ -60,6 +68,93 @@ class TestVCS(tests.TestCase):
 		self.assertEqual(VCS._detect_in_folder(subdir), ('git', subroot))
 
 
+@tests.slowTest
+@tests.skipUnless(
+	any(
+		map(VCS.check_dependencies, (VCS.BZR, VCS.GIT, VCS.HG))
+	), 'Missing dependencies')
+class TestMainWindowExtension(tests.TestCase):
+
+	def runTest(self):
+		plugin = VersionControlPlugin()
+
+		dir = get_tmp_dir('versioncontrol_TestMainWindowExtension')
+		notebook = tests.new_files_notebook(dir)
+		ui = setupGtkInterface(self, notebook=notebook)
+		plugin.extend(notebook)
+		plugin.extend(ui.mainwindow)
+
+		notebook_ext = plugin.get_extension(NotebookExtension)
+		self.assertIsInstance(notebook_ext, NotebookExtension)
+
+		window_ext = plugin.get_extension(MainWindowExtension)
+		self.assertIsInstance(window_ext, MainWindowExtension)
+
+		## init & save version
+		self.assertIsNone(notebook_ext.vcs)
+
+		def init(dialog):
+			self.assertIsInstance(dialog, VersionControlInitDialog)
+			choice = dialog.combobox.get_active_text()
+			self.assertTrue(choice and not choice.isspace())
+			dialog.emit('response', gtk.RESPONSE_YES)
+
+		with tests.DialogContext(init, SaveVersionDialog):
+			window_ext.save_version()
+
+		self.assertIsNotNone(notebook_ext.vcs)
+
+		window_ext._autosave_thread.join()
+		self.assertFalse(notebook_ext.vcs.modified)
+
+		## save version again
+		page = notebook.get_page(Path('Foo'))
+		page.parse('wiki', 'foo!')
+		notebook.store_page(page)
+
+		self.assertTrue(notebook_ext.vcs.modified)
+
+		with tests.DialogContext(SaveVersionDialog):
+			window_ext.save_version()
+
+		window_ext._autosave_thread.join()
+
+		self.assertFalse(notebook_ext.vcs.modified)
+
+		## show versions
+		with tests.DialogContext(VersionsDialog):
+			window_ext.show_versions()
+
+		## auto-save
+		plugin.preferences['autosave'] = True
+
+		page = notebook.get_page(Path('Fooooo'))
+		page.parse('wiki', 'foo!')
+		notebook.store_page(page)
+
+		self.assertTrue(notebook_ext.vcs.modified)
+		ui.emit('quit')
+		self.assertFalse(notebook_ext.vcs.modified)
+
+
+@tests.slowTest
+class TestVersionsDialog(tests.TestCase):
+
+	def testSideBySide(self):
+		app = get_side_by_side_app()
+		if Application('meld').tryexec():
+			self.assertIsNotNone(app)
+
+		if app is None:
+			print '\nCould not find an application for side-by-side comparison'
+		else:
+			self.assertTrue(app.tryexec)
+
+	def testDialog(self):
+		pass # TODO test other dialog functions
+
+
+
 #####################################################
 #
 # BAZAAR BACKEND TEST
@@ -80,7 +175,7 @@ class TestBazaar(tests.TestCase):
 		print '\n!! Some raw output from Bazaar expected here !!'
 
 		root = get_tmp_dir('versioncontrol_TestBazaar')
-		vcs = VCS.create(VCS.BZR, root)
+		vcs = VCS.create(VCS.BZR, root, root)
 		vcs.init()
 
 		#~ for notebookdir in (root, root.subdir('foobar')):
@@ -137,7 +232,7 @@ added:
 		self.assertEqual(vcs.get_diff(), ['=== No Changes\n'])
 
 		file.write('foo\nbaz\n')
-		vcs.commit_async('test 2')
+		vcs.commit('test 2')
 		diff = vcs.get_diff(versions=(1, 2))
 		diff = ''.join(filter(ignorelines, diff))
 		self.assertEqual(diff, '''\
@@ -215,7 +310,7 @@ class TestGit(tests.TestCase):
 		print '\n!! Some raw output from Git could appear here !!'
 
 		root = get_tmp_dir('versioncontrol_TestGit')
-		vcs = VCS.create(VCS.GIT, root)
+		vcs = VCS.create(VCS.GIT, root, root)
 		vcs.init()
 
 		#~ for notebookdir in (root, root.subdir('foobar')):
@@ -226,19 +321,10 @@ class TestGit(tests.TestCase):
 		subdir = root.subdir('foo/bar')
 		file = subdir.file('baz.txt')
 		file.write('foo\nbar\n')
-		self.assertEqual(''.join(vcs.get_status()), '''\
-# On branch master
-#
-# Initial commit
-#
-# Changes to be committed:
-#   (use "git rm --cached <file>..." to unstage)
-#
-#	new file:   .gitignore
-#	new file:   foo/bar/baz.txt
-#
-''' )
-
+		self.assertEqual(''.join(vcs.get_status(porcelain=True)),
+			'A  .gitignore\n'
+			'A  foo/bar/baz.txt\n'
+		)
 		vcs.update_staging()
 		vcs.commit('test 1')
 #[master 0f4132e] test 1
@@ -251,14 +337,9 @@ class TestGit(tests.TestCase):
 		file = subdir.file('bar.txt')
 		file.write('second\ntest\n')
 
-		self.assertEqual(''.join(vcs.get_status()), '''\
-# On branch master
-# Changes to be committed:
-#   (use "git reset HEAD <file>..." to unstage)
-#
-#	new file:   foo/bar/bar.txt
-#
-''' )
+		self.assertEqual(''.join(vcs.get_status(porcelain=True)),
+			'A  foo/bar/bar.txt\n'
+		)
 
 		vcs.update_staging()
 		vcs.commit('test 2')
@@ -300,19 +381,10 @@ diff --git a/foo/bar/bar.txt b/foo/bar/bar.txt
 ''' )
 
 		vcs.revert()
-		self.assertIn(vcs.get_status(), (
-[
-'# On branch master\n',
-'nothing to commit (working directory clean)\n'
-],
-[
-'# On branch master\n',
-'nothing to commit, working directory clean\n'
-],
-) )
+		self.assertEqual(vcs.get_status(porcelain=True), [])
 
 		file.write('second\nbaz\n')
-		vcs.commit_async('test 3')
+		vcs.commit('test 3')
 		diff = vcs.get_diff(versions=('HEAD', 'HEAD^'))
 		diff = ''.join(filter(ignorelines, diff))
 		self.assertEqual(diff, '''\
@@ -417,7 +489,7 @@ class TestMercurial(tests.TestCase):
 		print '\n!! Some raw output from Mercurial expected here !!'
 
 		root = get_tmp_dir('versioncontrol_TestMercurial')
-		vcs = VCS.create(VCS.HG, root)
+		vcs = VCS.create(VCS.HG, root, root)
 		vcs.init()
 
 		#~ for notebookdir in (root, root.subdir('foobar')):
@@ -456,7 +528,7 @@ diff --git a/foo/bar/baz.txt b/foo/bar/baz.txt
 
 
 		file.write('foo\nbaz\n')
-		vcs.commit_async('test 2')
+		vcs.commit('test 2')
 		diff = vcs.get_diff(versions=(0, 1))
 		diff = ''.join(filter(ignorelines, diff))
 		self.assertEqual(diff, '''\
@@ -472,10 +544,10 @@ diff --git a/foo/bar/baz.txt b/foo/bar/baz.txt
 		self.assertTrue(len(versions) == 2)
 		self.assertTrue(len(versions[0]) == 4)
 		self.assertEqual(versions[0][0], str(0))
-		self.assertEqual(versions[0][3], u'test 1\n')
+		self.assertEqual(versions[0][3], u'test 1')
 		self.assertTrue(len(versions[1]) == 4)
 		self.assertEqual(versions[1][0], str(1))
-		self.assertEqual(versions[1][3], u'test 2\n')
+		self.assertEqual(versions[1][3], u'test 2')
 
 
 		lines = vcs.get_version(file, version=0)
