@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2009 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2009,2014 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''This module contains helper classes for running external applications.
 
@@ -18,13 +18,19 @@ import gobject
 import zim.fs
 import zim.errors
 
+from zim.fs import File
 from zim.parsing import split_quoted_strings, is_uri_re, is_win32_path_re
-from zim.config import get_environ_list
+from zim.environ import environ
 
 
 logger = logging.getLogger('zim.applications')
 
 
+def _main_is_frozen():
+	# Detect whether we are running py2exe compiled version
+	return hasattr(sys, 'frozen') and sys.frozen
+
+
 class ApplicationError(zim.errors.Error):
 	'''Error raises for error in sub process errors'''
 
@@ -62,7 +68,7 @@ class Application(object):
 
 	STATUS_OK = 0 #: return code when the command executed succesfully
 
-	def __init__(self, cmd, tryexeccmd=None):
+	def __init__(self, cmd, tryexeccmd=None, encoding=None):
 		'''Constructor
 
 		@param cmd: the command for the external application, either a
@@ -70,6 +76,8 @@ class Application(object):
 		and arguments
 		@param tryexeccmd: command to check in L{tryexec()} as string.
 		If C{None} will default to C{cmd} or the first item of C{cmd}.
+		@param encoding: the encoding to use for commandline args
+		if known, else falls back to system default
 		'''
 		if isinstance(cmd, basestring):
 			cmd = split_quoted_strings(cmd)
@@ -78,6 +86,9 @@ class Application(object):
 		assert tryexeccmd is None or isinstance(tryexeccmd, basestring)
 		self.cmd = tuple(cmd)
 		self.tryexeccmd = tryexeccmd
+		self.encoding = encoding or zim.fs.ENCODING
+		if self.encoding == 'mbcs':
+			self.encoding = 'utf-8'
 
 	def __repr__(self):
 		if hasattr(self, 'key'):
@@ -101,8 +112,8 @@ class Application(object):
 				return None
 		elif os.name == 'nt':
 			# Check executable extensions from windows environment
-			extensions = get_environ_list('PATHEXT', '.com;.exe;.bat;.cmd')
-			for dir in get_environ_list('PATH'):
+			extensions = environ.get_list('PATHEXT', '.com;.exe;.bat;.cmd')
+			for dir in environ.get_list('PATH'):
 				for ext in extensions:
 					file = os.sep.join((dir, cmd + ext))
 					if zim.fs.isfile(file) and os.access(file, os.X_OK):
@@ -111,7 +122,7 @@ class Application(object):
 				return None
 		else:
 			# On POSIX no extension is needed to make scripts executable
-			for dir in get_environ_list('PATH'):
+			for dir in environ.get_list('PATH'):
 				file = os.sep.join((dir, cmd))
 				if zim.fs.isfile(file) and os.access(file, os.X_OK):
 					return file
@@ -139,12 +150,19 @@ class Application(object):
 		assert args is None or isinstance(args, (tuple, list))
 		argv = self._cmd(args)
 
+		# Expand home dir
+		if argv[0].startswith('~'):
+			cmd = File(argv[0]).path
+			argv = list(argv)
+			argv[0] = cmd
+
 		# if it is a python script, insert interpreter as the executable
-		if argv[0].endswith('.py') and sys.executable:
+		if argv[0].endswith('.py') and not _main_is_frozen():
 			argv = list(argv)
 			argv.insert(0, sys.executable)
+		# TODO: consider an additional commandline arg to re-use compiled python interpreter
 
-		argv = [a.encode(zim.fs.ENCODING) for a in argv]
+		argv = [a.encode(self.encoding) for a in argv]
 		if cwd:
 			cwd = unicode(cwd).encode(zim.fs.ENCODING)
 		return cwd, argv
@@ -212,9 +230,12 @@ class Application(object):
 			logger.warn(stderr)
 			# TODO: allow user to get this error as well - e.g. for logging image generator cmd
 
-		return map(unicode, [line + '\n' for line in stdout.splitlines()])
-			# Explicit newline conversion, e.g. on windows \r\n -> \n
-			# FIXME Assume local encoding is respected (!?)
+		# Explicit newline conversion, e.g. on windows \r\n -> \n
+		# FIXME Assume local encoding is respected (!?)
+		text = [unicode(line + '\n', errors='replace') for line in stdout.splitlines()]
+		if text and text[-1].endswith('\n') and not stdout.endswith('\n'):
+			text[-1] = text[-1][:-1] # strip additional \n
+		return text
 
 	def spawn(self, args=None, callback=None, data=None, cwd=None):
 		'''Start the application in the background and return immediately.
