@@ -23,6 +23,7 @@
 # 2010-06-29 1st working version
 #
 # TODO:
+# [ ] Better management of queue - how to avoid flooding it ?
 # [ ] Action for deleting files in context menu
 # [ ] Copy / cut files in context menu
 # [ ] Button to clean up the folder - only show when the folder is empty
@@ -42,6 +43,8 @@ import re
 import hashlib # for thumbfilenames
 import datetime
 import logging
+import Queue
+import threading
 
 import gobject
 import gtk
@@ -61,7 +64,6 @@ from zim.actions import toggle_action
 from zim.fs import File, Dir, format_file_size
 from zim.errors import Error
 from zim.applications import Application
-from zim.async import AsyncOperation
 from zim.parsing import url_encode, URL_ENCODE_READABLE
 
 from zim.gui.widgets import Button, BOTTOM_PANE, PANE_POSITIONS, \
@@ -275,7 +277,6 @@ class MainWindowExtension(WindowExtension):
 	def __init__(self, plugin, window):
 		WindowExtension.__init__(self, plugin, window)
 		self.preferences = plugin.preferences
-		self.uistate = plugin.uistate
 		self._monitor = None
 
 		# Init statusbar button
@@ -297,8 +298,9 @@ class MainWindowExtension(WindowExtension):
 		opener = self.window.get_resource_opener()
 		self.widget = AttachmentBrowserPluginWidget(self, opener, self.preferences)
 			# FIXME FIXME FIXME - get rid of ui object here
-		self.connectto(plugin, 'preferences-changed')
-		self.on_preferences_changed()
+
+		self.on_preferences_changed(plugin.preferences)
+		self.connectto(plugin.preferences, 'changed', self.on_preferences_changed)
 
 		# XXX
 		if self.window.ui.page:
@@ -307,12 +309,15 @@ class MainWindowExtension(WindowExtension):
 
 		self.connectto(self.window, 'pane-state-changed')
 
-	def on_preferences_changed(self):
+	def on_preferences_changed(self, preferences):
+		if self.widget is None:
+			return
+
 		try:
 			self.window.remove(self.widget)
 		except ValueError:
 			pass
-		self.window.add_tab(self.TAB_NAME, self.widget, self.preferences['pane'])
+		self.window.add_tab(self.TAB_NAME, self.widget, preferences['pane'])
 		self.widget.show_all()
 
 	@toggle_action(
@@ -393,14 +398,13 @@ class MainWindowExtension(WindowExtension):
 				n += 1
 		return n
 
-	def destroy(self):
+	def teardown(self):
 		self._disconnect_monitor()
 		self.toggle_fileview(False)
+		self.window.remove(self.widget)
 		if self.statusbar_frame:
 			self.window.statusbar.remove(self.statusbar_frame)
-
-		WindowExtension.destroy(self)
-
+		self.widget = None
 
 
 class uistate_property(object):
@@ -828,7 +832,8 @@ class ThumbnailManager(gobject.GObject):
 		gobject.GObject.__init__(self)
 
 		self.preferences = preferences
-		self.async_queue = []
+		self.async_thread = None
+		self.async_queue = Queue.Queue()
 
 		for dir in (
 			LOCAL_THUMB_STORAGE_NORMAL,
@@ -841,7 +846,11 @@ class ThumbnailManager(gobject.GObject):
 				pass
 
 	def clear_async_queue(self):
-		self.async_queue = []
+		if self.async_thread:
+			self.async_thread.stop()
+
+		self.async_thread = None
+		self.async_queue = Queue.Queue()
 
 	def get_thumbnail(self, file, size):
 		'''Get a C{Pixbuf} with the thumbnail for a given file
@@ -922,30 +931,18 @@ class ThumbnailManager(gobject.GObject):
 		if pixbuf:
 			return pixbuf
 		else:
-			self.async_queue.append( (file, thumbfile, size) )
-			if len(self.async_queue) == 1: # was empty
-				self._start_async_operation()
+			self.async_queue.put( (file, thumbfile, size) )
 
-			# TODO - allow multiple async threads at once, but have max
-			# use queue to deal with surplus requests ?
+			if not self.async_thread \
+			or not self.async_thread.is_alive():
+				self.async_thread = WorkerThread(self.async_queue, self._create_thumbnail_async)
+				self.async_thread.start()
 
-	def _start_async_operation(self):
-		args = self.async_queue[0]
-		operation = AsyncOperation(
-			self._create_thumbnail, args=args, callback=self._async_callback, data=self.async_queue[0])
-		operation.start()
-
-	def _async_callback(self, pixbuf, error, exc_info, data):
-			# Callback is called from main tread, in idle event
-			# so it is allowed to kick off new async operations
-			if error:
-				logger.error('Error while creating thumbnail', exc_info=exc_info)
-
-			if self.async_queue:
-				self.async_queue.pop(0)
-
-			if self.async_queue:
-				self._start_async_operation()
+	def _create_thumbnail_async(self, file, thumbfile, size):
+		try:
+			self._create_thumbnail(file, thumbfile, size)
+		except:
+			logger.exception('Error while creating thumbnail')
 
 	def get_thumbnail_file(self, file, size):
 		'''Get L{File} object for thumbnail
@@ -985,6 +982,26 @@ class ThumbnailManager(gobject.GObject):
 gobject.type_register(ThumbnailManager)
 
 
+class WorkerThread(threading.Thread):
+
+	daemon = True
+
+	def __init__(self, queue, func):
+		threading.Thread.__init__(self)
+		self.queue = queue
+		self.func = func
+		self._stop = False
+
+	def stop(self):
+		self._stop = True
+
+	def run(self):
+		while not self._stop:
+			args = self.queue.get()
+			self.func(*args)
+			self.queue.task_done()
+
+
 class Thumbnailer(object):
 
 	def create_thumbnail(self, file, thumbfile, size):
