@@ -14,40 +14,20 @@
 # Copyright 2009 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 #
 
-import gtk
 import glob
+import re
 
+from zim.plugins.base.imagegenerator import ImageGeneratorPlugin, ImageGeneratorClass
 from zim.fs import File, TmpFile
-from zim.plugins import PluginClass
 from zim.config import data_file
-from zim.templates import GenericTemplate
+from zim.templates import get_template
 from zim.applications import Application
-from zim.gui.imagegeneratordialog import ImageGeneratorClass, ImageGeneratorDialog
-from zim.gui.widgets import populate_popup_add_separator
 
 # TODO put these commands in preferences
 gnu_r_cmd = ('R',)
 
-ui_xml = '''
-<ui>
-	<menubar name='menubar'>
-		<menu action='insert_menu'>
-			<placeholder name='plugin_items'>
-				<menuitem action='insert_gnu_r_plot'/>
-			</placeholder>
-		</menu>
-	</menubar>
-</ui>
-'''
 
-ui_actions = (
-	# name, stock id, label, accelerator, tooltip, read only
-	('insert_gnu_r_plot', None, _('GNU _R Plot...'), '', '', False),
-		# T: menu item for insert plot plugin
-)
-
-
-class InsertGNURPlotPlugin(PluginClass):
+class InsertGNURPlotPlugin(ImageGeneratorPlugin):
 
 	plugin_info = {
 		'name': _('Insert GNU R Plot'), # T: plugin name
@@ -58,77 +38,61 @@ This plugin provides a plot editor for zim based on GNU R.
 		'author': 'Lee Braiden',
 	}
 
+	object_type = 'gnu_r_plot'
+	short_label = _('GNU _R Plot') # T: menu item
+	insert_label = _('Insert GNU R Plot') # T: menu item
+	edit_label = _('_Edit GNU R Plot') # T: menu item
+	syntax = 'r'
+
 	@classmethod
 	def check_dependencies(klass):
 		has_gnur = Application(gnu_r_cmd).tryexec()
 		return has_gnur, [('GNU R', has_gnur, True)]
 
-	def __init__(self, ui):
-		PluginClass.__init__(self, ui)
-		if self.ui.ui_type == 'gtk':
-			self.ui.add_actions(ui_actions, self)
-			self.ui.add_ui(ui_xml, self)
-			self.register_image_generator_plugin('gnu_r_plot')
-
-	def insert_gnu_r_plot(self):
-		dialog = InsertGNURPlotDialog.unique(self, self.ui)
-		dialog.run()
-
-	def edit_object(self, buffer, iter, image):
-		dialog = InsertGNURPlotDialog(self.ui, image=image)
-		dialog.run()
-
-	def do_populate_popup(self, menu, buffer, iter, image):
-		populate_popup_add_separator(menu, prepend=True)
-
-		item = gtk.MenuItem(_('_Edit GNU R Plot')) # T: menu item in context menu
-		item.connect('activate',
-			lambda o: self.edit_object(buffer, iter, image))
-		menu.prepend(item)
-
-
-
-class InsertGNURPlotDialog(ImageGeneratorDialog):
-
-	def __init__(self, ui, image=None):
-		generator = GNURPlotGenerator()
-		ImageGeneratorDialog.__init__(self, ui, _('GNU R Plot'), # T: dialog title
-			generator, image, help=':Plugins:GNU R Plot Editor' )
-
 
 class GNURPlotGenerator(ImageGeneratorClass):
 
 	uses_log_file = False
 
-	type = 'gnu_r_plot'
+	object_type = 'gnu_r_plot'
 	scriptname = 'gnu_r_plot.r'
 	imagename = 'gnu_r_plot.png'
 
-	def __init__(self):
-		file = data_file('templates/plugins/gnu_r_editor.r')
-		assert file, 'BUG: could not find templates/plugins/gnu_r_editor.r'
-		self.template = GenericTemplate(file.readlines(), name=file)
+	def __init__(self, plugin):
+		ImageGeneratorClass.__init__(self, plugin)
+		self.template = get_template('plugins', 'gnu_r_editor.r')
 		self.plotscriptfile = TmpFile(self.scriptname)
 
 	def generate_image(self, text):
-		if isinstance(text, basestring):
-			text = text.splitlines(True)
 
 		plotscriptfile = self.plotscriptfile
 		pngfile = File(plotscriptfile.path[:-2] + '.png')
 
 		plot_script = "".join(text)
+		
+		plot_width = 480 # default image width (px)
+		plot_height = 480 # default image height (px)
+
+		# LOOK for image size in comments of the script
+		r=re.search(r"^#\s*WIDTH\s*=\s*([0-9]+)$",plot_script,re.M)
+		if r:
+			plot_width=int(r.group(1))
+		r=re.search(r"^#\s*HEIGHT\s*=\s*([0-9]+)$",plot_script,re.M)
+		if r:
+			plot_height=int(r.group(1))
 
 		template_vars = {
 			'gnu_r_plot_script': plot_script,
+			'r_width': plot_width,
+			'r_height': plot_height,
 			'png_fname': pngfile.path.replace('\\', '/'),
 				# Even on windows, GNU R expects unix path seperator
 		}
 
 		# Write to tmp file usign the template for the header / footer
-		plotscriptfile.writelines(
-			self.template.process(template_vars)
-		)
+		lines = []
+		self.template.process(lines, template_vars)
+		plotscriptfile.writelines(lines)
 		#print '>>>%s<<<' % plotscriptfile.read()
 
 		# Call GNU R
