@@ -12,16 +12,14 @@
 #
 #
 
-import gtk
 import glob
 
+from zim.plugins.base.imagegenerator import ImageGeneratorPlugin, ImageGeneratorClass
 from zim.fs import File, TmpFile
-from zim.plugins import PluginClass
 from zim.config import data_file
-from zim.templates import GenericTemplate
+from zim.templates import get_template
 from zim.applications import Application, ApplicationError
-from zim.gui.imagegeneratordialog import ImageGeneratorClass, ImageGeneratorDialog
-from zim.gui.widgets import populate_popup_add_separator
+
 
 # TODO put these commands in preferences
 lilypond_cmd = ('lilypond', '-ddelete-intermediate-files',
@@ -30,23 +28,6 @@ lilypond_cmd = ('lilypond', '-ddelete-intermediate-files',
 convertly_cmd = ('convert-ly', '--current-version', '--edit')
 lilypondver_cmd = ('lilypond', '--version')
 
-ui_xml = '''
-<ui>
-	<menubar name='menubar'>
-		<menu action='insert_menu'>
-			<placeholder name='plugin_items'>
-				<menuitem action='insert_score'/>
-			</placeholder>
-		</menu>
-	</menubar>
-</ui>
-'''
-
-ui_actions = (
-	# name, stock id, label, accelerator, tooltip, read only
-	('insert_score', None, _('S_core...'), '', _('Insert score'), False),
-		# T: menu item for insert score plugin
-)
 
 def _get_lilypond_version():
 	try:
@@ -56,7 +37,7 @@ def _get_lilypond_version():
 	except ApplicationError:
 		return '2.14.2'
 
-class InsertScorePlugin(PluginClass):
+class InsertScorePlugin(ImageGeneratorPlugin):
 
 	plugin_info = {
 		'name': _('Insert Score'), # T: plugin name
@@ -75,63 +56,32 @@ This is a core plugin shipping with zim.
 		('include_footer', 'string', _('Common include footer'), ''), # T: plugin preference
 	]
 
+	object_type = 'score'
+	short_label = _('S_core') # T: menu item
+	insert_label = _('Insert Score') # T: menu item
+	edit_label = _('_Edit Score') # T: menu item
+	syntax = None
+
 	@classmethod
 	def check_dependencies(klass):
 		has_lilypond = Application(lilypond_cmd).tryexec()
 		return has_lilypond, [('GNU Lilypond', has_lilypond, True)]
 
-	def __init__(self, ui):
-		PluginClass.__init__(self, ui)
-		if self.ui.ui_type == 'gtk':
-			self.ui.add_actions(ui_actions, self)
-			self.ui.add_ui(ui_xml, self)
-			self.register_image_generator_plugin('score')
-
-	def insert_score(self):
-		dialog = InsertScoreDialog.unique(self, self.ui, preferences=self.preferences)
-		dialog.run()
-
-	def edit_object(self, buffer, iter, image):
-		dialog = InsertScoreDialog(self.ui, image=image, preferences=self.preferences)
-		dialog.run()
-
-	def do_populate_popup(self, menu, buffer, iter, image):
-		populate_popup_add_separator(menu, prepend=True)
-
-		item = gtk.MenuItem(_('_Edit Score')) # T: menu item in context menu
-		item.connect('activate',
-			lambda o: self.edit_object(buffer, iter, image))
-		menu.prepend(item)
-
-
-
-class InsertScoreDialog(ImageGeneratorDialog):
-
-	def __init__(self, ui, preferences={}, image=None):
-		generator = ScoreGenerator(preferences=preferences)
-		ImageGeneratorDialog.__init__(self, ui, _('Insert Score'), # T: dialog title
-			generator, image, help=':Plugins:Score Editor', syntax="lilypond" )
-
 
 class ScoreGenerator(ImageGeneratorClass):
 
-	type = 'score'
+	object_type = 'score'
 	scriptname = 'score.ly'
 	imagename = 'score.png'
 	cur_lilypond_version = None
-	include_header = ''
-	include_footer = ''
 
-	def __init__(self, preferences={}):
-		file = data_file('templates/plugins/scoreeditor.ly')
-		assert file, 'BUG: could not find templates/plugins/scoreeditor.ly'
-		self.template = GenericTemplate(file.readlines(), name=file)
+	def __init__(self, plugin):
+		ImageGeneratorClass.__init__(self, plugin)
+		self.template = get_template('plugins', 'scoreeditor.ly')
 		self.scorefile = TmpFile(self.scriptname)
 		self.cur_lilypond_version = _get_lilypond_version()
-		if preferences.has_key('include_header'):
-			self.include_header = preferences['include_header']
-		if preferences.has_key('include_footer'):
-			self.include_footer = preferences['include_footer']
+		self.include_header = plugin.preferences['include_header']
+		self.include_footer = plugin.preferences['include_footer']
 
 	def process_input(self, text):
 		'''Prepend version string to user input. It is also stored in
@@ -156,8 +106,6 @@ class ScoreGenerator(ImageGeneratorClass):
 		return (version, outtext)
 
 	def generate_image(self, text):
-		if isinstance(text, basestring):
-			text = text.splitlines(True)
 
 		(version, text) = self.extract_version(text)
 		text = ''.join(text)
@@ -165,11 +113,14 @@ class ScoreGenerator(ImageGeneratorClass):
 
 		# Write to tmp file using the template for the header / footer
 		scorefile = self.scorefile
-		scorefile.writelines(
-			self.template.process({'score': text,
-				'version': version,
-				'include_header': self.include_header,
-				'include_footer': self.include_footer}) )
+		lines = []
+		self.template.process(lines, {
+			'score': text,
+			'version': version or '',
+			'include_header': self.include_header or '',
+			'include_footer': self.include_footer or '',
+		} )
+		scorefile.writelines(lines)
 		#~ print '>>>%s<<<' % scorefile.read()
 
 		# Call convert-ly to convert document of current version of
