@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # Copyright 2010 Fabian Moser
-# Copyright 2011, 2012 Jaap Karssenberg
+# Copyright 2011-2014 Jaap Karssenberg
 
 
 import gobject
@@ -10,12 +10,12 @@ import pango
 
 import logging
 
-from zim.plugins import PluginClass
+from zim.plugins import PluginClass, extends, WindowExtension
 from zim.gui.pageindex import PageTreeStore, PageTreeIter, PageTreeView, \
-	NAME_COL, PATH_COL, EMPTY_COL, STYLE_COL, FGCOLOR_COL, WEIGHT_COL, N_CHILD_COL
+	NAME_COL, PATH_COL, EMPTY_COL, STYLE_COL, FGCOLOR_COL, WEIGHT_COL, N_CHILD_COL, TIP_COL
 from zim.notebook import Path
 from zim.index import IndexPath, IndexTag
-from zim.gui.widgets import LEFT_PANE, PANE_POSITIONS, populate_popup_add_separator, ScrolledWindow
+from zim.gui.widgets import LEFT_PANE, PANE_POSITIONS, populate_popup_add_separator, ScrolledWindow, encode_markup_text
 from zim.gui.clipboard import pack_urilist, INTERNAL_PAGELIST_TARGET_NAME
 from zim.signals import ConnectorMixin
 
@@ -23,6 +23,174 @@ from zim.signals import ConnectorMixin
 logger = logging.getLogger('zim.plugins.tags')
 
 
+
+class TagsPlugin(PluginClass):
+
+	plugin_info = {
+		'name': _('Tags'), # T: plugin name
+		'description': _('''\
+This plugin provides a page index filtered by means of selecting tags in a cloud.
+'''), # T: plugin description
+		'author': 'Fabian Moser',
+		'help': 'Plugins:Tags',
+	}
+
+	plugin_preferences = (
+		# key, type, label, default
+		('pane', 'choice', _('Position in the window'), LEFT_PANE, PANE_POSITIONS),
+			# T: option for plugin preferences
+	)
+
+
+@extends('MainWindow')
+class MainWindowExtension(WindowExtension):
+
+	def __init__(self, plugin, window):
+		WindowExtension.__init__(self, plugin, window)
+
+		self.widget = TagsPluginWidget(self.window.ui.notebook.index, self.uistate, self.window.ui) # XXX
+
+		self.on_preferences_changed(plugin.preferences)
+		self.connectto(plugin.preferences, 'changed', self.on_preferences_changed)
+
+		self.uistate.setdefault('vpane_pos', 150)
+		self.widget.set_position(self.uistate['vpane_pos'])
+		def update_uistate(*a):
+			self.uistate['vpane_pos'] = self.widget.get_position()
+		self.widget.connect('notify::position', update_uistate)
+
+	def on_preferences_changed(self, preferences):
+		if self.widget is None:
+			return
+
+		try:
+			self.window.remove(self.widget)
+		except ValueError:
+			pass
+		self.window.add_tab(_('Tags'), self.widget, preferences['pane'])
+		self.widget.show_all()
+
+	def teardown(self):
+		self.window.remove(self.widget)
+		self.widget.disconnect_all()
+		self.widget = None
+
+
+class TagsPluginWidget(ConnectorMixin, gtk.VPaned):
+	'''Widget combining a tag cloud and a tag based page treeview'''
+
+	def __init__(self, index, uistate, ui): # XXX
+		gtk.VPaned.__init__(self)
+		self.index = index
+		self.uistate = uistate
+
+		self.uistate.setdefault('treeview', 'tagged', set(['tagged', 'tags']))
+		self.uistate.setdefault('tagcloud_sorting', 'score', set(['alpha', 'score']))
+		self.uistate.setdefault('show_full_page_name', True)
+
+		self.tagcloud = TagCloudWidget(self.index, sorting=self.uistate['tagcloud_sorting'])
+		self.pack1(ScrolledWindow(self.tagcloud), shrink=False)
+
+		self.treeview = TagsPageTreeView(ui) # XXX
+		self._treeview_mode = (None, None)
+		self.pack2(ScrolledWindow(self.treeview), shrink=False)
+
+		self.treeview.connect('populate-popup', self.on_populate_popup)
+		self.tagcloud.connect('selection-changed', self.on_cloud_selection_changed)
+		self.tagcloud.connect('sorting-changed', self.on_cloud_sortin_changed)
+
+		self.connectto_all(ui, ( # XXX
+			'open-page',
+			('start-index-update', lambda o: self.disconnect_model()),
+			('end-index-update', lambda o: self.reload_model()),
+		))
+
+		self.reload_model()
+
+	def on_open_page(self, ui, page, path):
+		self.treeview.select_page(path)
+
+	def toggle_treeview(self):
+		'''Toggle the treeview type in the widget'''
+		if self.uistate['treeview'] == 'tagged':
+			self.uistate['treeview'] = 'tags'
+		else:
+			self.uistate['treeview'] = 'tagged'
+
+		self.reload_model()
+
+	def toggle_show_full_page_name(self):
+		self.uistate['show_full_page_name'] = not self.uistate['show_full_page_name']
+		self.reload_model()
+
+	def on_populate_popup(self, treeview, menu):
+		# Add a popup menu item to switch the treeview mode
+		populate_popup_add_separator(menu, prepend=True)
+
+		item = gtk.CheckMenuItem(_('Show full page name')) # T: menu option
+		item.set_active(self.uistate['show_full_page_name'])
+		item.connect_object('toggled', self.__class__.toggle_show_full_page_name, self)
+		menu.prepend(item)
+
+		item = gtk.CheckMenuItem(_('Sort pages by tags')) # T: menu option
+		item.set_active(self.uistate['treeview'] == 'tags')
+		item.connect_object('toggled', self.__class__.toggle_treeview, self)
+		menu.prepend(item)
+
+		menu.show_all()
+
+	def on_cloud_selection_changed(self, cloud):
+		filter = cloud.get_tag_filter()
+		type, was_filtered = self._treeview_mode
+		is_filtered = (filter is not None)
+		if type == 'tagged' and was_filtered != is_filtered:
+			# Switch between tag view and normal index or vice versa
+			self._reload_model(type, filter)
+		else:
+			self.treeview.set_tag_filter(filter)
+
+	def on_cloud_sortin_changed(self, cloud, sorting):
+		self.uistate['tagcloud_sorting'] = sorting
+
+	def disconnect_model(self):
+		'''Stop the model from listening to the index. Used to
+		unhook the model before reloading the index. Typically
+		should be followed by reload_model().
+		'''
+		self.treeview.disconnect_index()
+		self.tagcloud.disconnect_index()
+
+	def reload_model(self):
+		'''Re-initialize the treeview model. This is called when
+		reloading the index to get rid of out-of-sync model errors
+		without need to close the app first.
+		'''
+		assert self.uistate['treeview'] in ('tagged', 'tags')
+
+		if self.tagcloud.index is None:
+			self.tagcloud.set_index(self.index)
+
+		type = self.uistate['treeview']
+		filter = self.tagcloud.get_tag_filter()
+		self._reload_model(type, filter)
+
+	def _reload_model(self, type, filter):
+		if type == 'tagged':
+			if filter is None:
+				model = DuplicatePageTreeStore(self.index)
+					# show the normal index in this case
+			else:
+				model = TaggedPageTreeStore(self.index, self.uistate['show_full_page_name'])
+		elif type == 'tags':
+			model = TagsPageTreeStore(self.index, self.uistate['show_full_page_name'])
+		else:
+			assert False
+
+		is_filtered = (filter is not None)
+		self._treeview_mode = (type, is_filtered)
+		self.treeview.set_model(model, filter)
+
+
 class PageTreeTagIter(object):
 	'''Simple wrapper for IndexTag objects used as tree iters
 
@@ -48,6 +216,7 @@ class DuplicatePageTreeStore(PageTreeStore):
 	'''
 
 	def select_page(self, path):
+		'''Since there may be duplicates of each page, highlight all of them'''
 		oldpath = self.selected_page
 		self.selected_page = path
 
@@ -86,8 +255,9 @@ class TagsPageTreeStore(DuplicatePageTreeStore):
 
 	filter_depth = 2 # tag filter applies to top two levels
 
-	def __init__(self, index):
+	def __init__(self, index, show_full_page_name=True):
 		self._reverse_cache = {}
+		self.show_full_page_name = show_full_page_name
 		self.untagged = IndexTag(_('untagged'), -1)
 			# T: label for untagged pages in side pane
 		PageTreeStore.__init__(self, index)
@@ -375,6 +545,8 @@ class TagsPageTreeStore(DuplicatePageTreeStore):
 			tag = iter.indextag
 			if column == NAME_COL:
 				return tag.name
+			elif column == TIP_COL:
+				return encode_markup_text(tag.name)
 			elif column == PATH_COL:
 				return tag
 			elif column == EMPTY_COL:
@@ -400,7 +572,15 @@ class TagsPageTreeStore(DuplicatePageTreeStore):
 				#~ else:
 					#~ return str(self.index.n_list_tagged_pages(tag))
 		else:
-			return PageTreeStore.on_get_value(self, iter, column)
+			if column == NAME_COL and self.show_full_page_name:
+				# Show top level pages with full contex
+				# top level tree is tags, so top level pages len(path) is 2
+				if len(iter.treepath) <= 2:
+					return iter.indexpath.name
+				else:
+					return iter.indexpath.basename
+			else:
+				return PageTreeStore.on_get_value(self, iter, column)
 
 
 class TaggedPageTreeStore(DuplicatePageTreeStore):
@@ -411,9 +591,10 @@ class TaggedPageTreeStore(DuplicatePageTreeStore):
 
 	filter_depth = 1 # tag filter only applies to top level
 
-	def __init__(self, index):
+	def __init__(self, index, show_full_page_name=True):
 		PageTreeStore.__init__(self, index)
 		self._reverse_cache = {}
+		self.show_full_page_name = show_full_page_name
 
 	def _connect(self):
 		def on_page_changed(o, path, signal):
@@ -522,6 +703,17 @@ class TaggedPageTreeStore(DuplicatePageTreeStore):
 		else:
 			return PageTreeStore.on_iter_n_children(self, iter)
 
+	def on_get_value(self, iter, column):
+		'''Returns the data for a specific column'''
+		if column == NAME_COL and self.show_full_page_name:
+			# Show top level pages with full contex
+			if len(iter.treepath) == 1:
+				return iter.indexpath.name
+			else:
+				return iter.indexpath.basename
+		else:
+			return PageTreeStore.on_get_value(self, iter, column)
+
 
 class TagsPageTreeView(PageTreeView):
 
@@ -630,6 +822,57 @@ class TagsPageTreeView(PageTreeView):
 		data = pack_urilist((link,))
 		selectiondata.set(INTERNAL_PAGELIST_TARGET_NAME, 8, data)
 
+	def select_page(self, path, vivificate=False):
+		'''Select a page in the treeview if the page is not already selected
+
+		@param path: a notebook L{Path} object for the page
+		@keyword vivificate: when C{True} the path is created
+		temporarily when it did not yet exist
+
+		@returns: a gtk TreePath (tuple of intergers) or C{None}
+		'''
+		#~ print '!! SELECT', path
+		model = self.get_model()
+		if model is None:
+			return None # index not yet initialized ...
+
+		# change selection only if necessary
+		selected_path = self.get_selected_path()
+		if path == selected_path:
+			_, iter = self.get_selection().get_selected()
+			treepath = model.get_path(iter)
+			logger.debug('Already selected: "%s"', treepath)
+		else:
+			treepath = model.get_treepath(path)
+			if treepath:
+				# path existed, now select it
+				self.select_treepath(treepath)
+			elif vivificate:
+				# path does not exist, but we can create it
+				path = model.index.touch(path)
+				treepath = model.get_treepath(path)
+				assert treepath, 'BUG: failed to touch placeholder'
+				self.select_treepath(treepath)
+			else:
+				# path does not exist and we are not going to create it
+				return None
+
+		rowreference = gtk.TreeRowReference(model, treepath)
+			# make reference before cleanup - path may change
+
+		if self._cleanup and self._cleanup.valid():
+			mytreepath = self._cleanup.get_path()
+			if mytreepath != treepath:
+				indexpath = model.get_indexpath( model.get_iter(mytreepath) )
+				#~ print '!! CLEANUP', indexpath
+				model.index.cleanup(indexpath)
+
+		self._cleanup = rowreference
+
+		model.select_page(path) # highlight in model
+
+		return treepath
+
 # Need to register classes defining gobject signals
 gobject.type_register(TagsPageTreeView)
 
@@ -668,7 +911,7 @@ class TagCloudWidget(ConnectorMixin, gtk.TextView):
 		'sorting-changed': (gobject.SIGNAL_RUN_LAST, None, (object,)),
 	}
 
-	def __init__(self, index=None, sorting='score'):
+	def __init__(self, index, sorting='score'):
 		gtk.TextView.__init__(self, None) # Create TextBuffer implicitly
 		self.set_name('zim-tags-tagcloud')
 		self.index = None
@@ -678,9 +921,7 @@ class TagCloudWidget(ConnectorMixin, gtk.TextView):
 		self.set_wrap_mode(gtk.WRAP_CHAR)
 
 		self.set_sorting(sorting)
-
-		if index:
-			self.set_index(index)
+		self.set_index(index)
 
 	def set_index(self, index):
 		'''Connect to an Index object'''
@@ -729,7 +970,11 @@ class TagCloudWidget(ConnectorMixin, gtk.TextView):
 	def _update(self, *a):
 		'''Update the cloud to show only tags that share a set of pages
 		with the selected tags.'''
-		selected = [b.indextag for b in self.get_children() if b.get_active()]
+		selected = [b.indextag for b in self.get_children()
+			if b.get_active() and
+			self.index.lookup_tag(b.indextag.name)]
+			# Need the lookup here in case the tag went missing in the
+			# mean time e.g. due to editing of the page
 		self._clear()
 
 		buffer = self.get_buffer()
@@ -779,172 +1024,4 @@ class TagCloudWidget(ConnectorMixin, gtk.TextView):
 gobject.type_register(TagCloudWidget)
 
 
-class TagsPluginWidget(ConnectorMixin, gtk.VPaned):
-	'''Widget combining a tag cloud and a tag based page treeview'''
 
-	def __init__(self, plugin):
-		gtk.VPaned.__init__(self)
-		self.plugin = plugin
-
-		self.plugin.uistate.setdefault('vpane_pos', 150)
-		self.plugin.uistate.setdefault('treeview', 'tagged', set(['tagged', 'tags']))
-		self.plugin.uistate.setdefault('tagcloud_sorting', 'score', set(['alpha', 'score']))
-
-		self.tagcloud = TagCloudWidget(sorting=self.plugin.uistate['tagcloud_sorting'])
-		self.pack1(ScrolledWindow(self.tagcloud), shrink=False)
-
-		self.treeview = TagsPageTreeView(self.plugin.ui)
-		self._treeview_mode = (None, None)
-		self.pack2(ScrolledWindow(self.treeview), shrink=False)
-
-		self.treeview.connect('populate-popup', self.on_populate_popup)
-		self.tagcloud.connect('selection-changed', self.on_cloud_selection_changed)
-		self.tagcloud.connect('sorting-changed', self.on_cloud_sortin_changed)
-
-		self.connectto_all(self.plugin.ui, (
-			'open-page',
-			('start-index-update', lambda o: self.disconnect_model()),
-			('end-index-update', lambda o: self.reload_model()),
-		))
-
-	def finalize_notebook(self, notebook):
-		self.tagcloud.set_sorting(self.plugin.uistate['tagcloud_sorting'])
-
-		self.set_position(self.plugin.uistate['vpane_pos'])
-		def update_uistate(*a):
-			self.plugin.uistate['vpane_pos'] = self.get_position()
-		self.connect('notify::position', update_uistate)
-
-		self.reload_model()
-
-	def on_open_page(self, ui, page, path):
-		self.treeview.select_page(path)
-
-	def toggle_treeview(self):
-		'''Toggle the treeview type in the widget'''
-		if self.plugin.uistate['treeview'] == 'tagged':
-			self.plugin.uistate['treeview'] = 'tags'
-		else:
-			self.plugin.uistate['treeview'] = 'tagged'
-
-		self.reload_model()
-
-	def on_populate_popup(self, treeview, menu):
-		# Add a popup menu item to switch the treeview mode
-		populate_popup_add_separator(menu, prepend=True)
-
-		item = gtk.CheckMenuItem(_('Sort pages by tags')) # T: menu option
-		item.set_active(self.plugin.uistate['treeview'] == 'tags')
-		item.connect_object('toggled', self.__class__.toggle_treeview, self)
-		menu.prepend(item)
-
-		menu.show_all()
-
-	def on_cloud_selection_changed(self, cloud):
-		filter = cloud.get_tag_filter()
-		type, was_filtered = self._treeview_mode
-		is_filtered = (filter is not None)
-		if type == 'tagged' and was_filtered != is_filtered:
-			# Switch between tag view and normal index or vice versa
-			self._reload_model(type, filter)
-		else:
-			self.treeview.set_tag_filter(filter)
-
-	def on_cloud_sortin_changed(self, cloud, sorting):
-		self.plugin.uistate['tagcloud_sorting'] = sorting
-
-	def disconnect_model(self):
-		'''Stop the model from listening to the index. Used to
-		unhook the model before reloading the index. Typically
-		should be followed by reload_model().
-		'''
-		self.treeview.disconnect_index()
-		self.tagcloud.disconnect_index()
-
-	def reload_model(self):
-		'''Re-initialize the treeview model. This is called when
-		reloading the index to get rid of out-of-sync model errors
-		without need to close the app first.
-		'''
-		assert self.plugin.uistate['treeview'] in ('tagged', 'tags')
-
-		if self.tagcloud.index is None:
-			self.tagcloud.set_index(self.plugin.ui.notebook.index)
-
-		type = self.plugin.uistate['treeview']
-		filter = self.tagcloud.get_tag_filter()
-		self._reload_model(type, filter)
-
-
-	def _reload_model(self, type, filter):
-		index = self.plugin.ui.notebook.index # FIXME clean up law of D
-		if type == 'tagged':
-			if filter is None:
-				model = DuplicatePageTreeStore(index)
-					# show the normal index in this case
-			else:
-				model = TaggedPageTreeStore(index)
-		elif type == 'tags':
-			model = TagsPageTreeStore(index)
-		else:
-			assert False
-
-		is_filtered = (filter is not None)
-		self._treeview_mode = (type, is_filtered)
-		self.treeview.set_model(model, filter)
-
-		if self.plugin.ui.page:
-			model.select_page(self.plugin.ui.page)
-
-
-class TagsPlugin(PluginClass):
-
-	plugin_info = {
-		'name': _('Tags'), # T: plugin name
-		'description': _('''\
-This plugin provides a page index filtered by means of selecting tags in a cloud.
-'''), # T: plugin description
-		'author': 'Fabian Moser',
-		'help': 'Plugins:Tags',
-	}
-
-	plugin_preferences = (
-		# key, type, label, default
-		('pane', 'choice', _('Position in the window'), LEFT_PANE, PANE_POSITIONS),
-			# T: option for plugin preferences
-	)
-
-	def __init__(self, ui):
-		PluginClass.__init__(self, ui)
-		self.sidepane_widget = None
-
-	def initialize_ui(self, ui):
-		if self.ui.ui_type == 'gtk':
-			self.connect_embedded_widget()
-
-	def finalize_notebook(self, notebook):
-		if self.ui.ui_type == 'gtk' and self.sidepane_widget:
-			self.sidepane_widget.finalize_notebook(notebook)
-
-	def destroy(self):
-		self.disconnect_embedded_widget()
-		PluginClass.destroy(self)
-
-	def do_preferences_changed(self):
-		if self.ui.ui_type == 'gtk':
-			self.connect_embedded_widget() # refresh pane position
-
-	def connect_embedded_widget(self):
-		if self.sidepane_widget is None:
-			self.sidepane_widget = TagsPluginWidget(self)
-		else:
-			self.ui.mainwindow.remove(self.sidepane_widget)
-
-		self.ui.mainwindow.add_tab(_('Tags'), self.sidepane_widget, self.preferences['pane'])
-		self.sidepane_widget.show_all()
-
-	def disconnect_embedded_widget(self):
-		if not self.sidepane_widget is None:
-			self.ui.mainwindow.remove(self.sidepane_widget)
-			self.sidepane_widget.disconnect_all()
-			self.sidepane_widget = None
