@@ -1,25 +1,28 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2009-2012 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2009-2014 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 # Copyright 2012 Damien Accorsi <damien.accorsi@free.fr>
 
 from __future__ import with_statement
 
+import gobject
 import gtk
 
 import os
 import logging
 
-from zim.fs import FS, File
-from zim.plugins import PluginClass
+from zim.fs import FS, File, TmpFile
+from zim.plugins import PluginClass, extends, WindowExtension, ObjectExtension
+from zim.actions import action
+from zim.signals import ConnectorMixin
 from zim.errors import Error
 from zim.applications import Application
-from zim.async import AsyncOperation
-from zim.config import value_is_coord
+from zim.gui.applications import DesktopEntryFile
+from zim.config import value_is_coord, data_dirs
 from zim.gui.widgets import ErrorDialog, QuestionDialog, Dialog, \
 	PageEntry, IconButton, SingleClickTreeView, \
 	ScrolledWindow, ScrolledTextView, VPaned
-from zim.utils import natural_sort_key
+from zim.utils import natural_sort_key, FunctionThread
 
 
 if os.environ.get('ZIM_TEST_RUNNING'):
@@ -38,37 +41,191 @@ else:
 logger = logging.getLogger('zim.plugins.versioncontrol')
 
 
-ui_xml = '''
-<ui>
-<menubar name='menubar'>
-	<menu action='file_menu'>
-		<placeholder name='versioning_actions'>
-			<menuitem action='save_version'/>
-			<menuitem action='show_versions'/>
-		</placeholder>
-	</menu>
-</menubar>
-</ui>
-'''
+class VersionControlPlugin(PluginClass):
+
+	plugin_info = {
+		'name': _('Version Control'), # T: plugin name
+		'description': _('''\
+This plugin adds version control for notebooks.
+
+This plugin supports the Bazaar, Git and Mercurial version control systems.
+
+This is a core plugin shipping with zim.
+'''), # T: plugin description
+		'author': 'Jaap Karssenberg & John Drinkwater & Damien Accorsi',
+		'help': 'Plugins:Version Control',
+	}
+
+	plugin_preferences = (
+		('autosave', 'bool', _('Autosave version on regular intervals'), False), # T: Label for plugin preference
+	)
+
+	@classmethod
+	def check_dependencies(klass):
+		has_bzr = VCS.check_dependencies(VCS.BZR)
+		has_git  = VCS.check_dependencies(VCS.GIT)
+		has_hg  = VCS.check_dependencies(VCS.HG)
+		#TODO parameterize the return, so that a new backend will be automatically available
+		return has_bzr|has_hg|has_git, [('bzr', has_bzr, False), ('hg', has_hg, False), ('git', has_git, False)]
+
+	def extend(self, obj):
+		name = obj.__class__.__name__
+		if name == 'MainWindow':
+			nb = obj.ui.notebook # XXX
+			nb_ext = self.get_extension(NotebookExtension, notebook=nb)
+			assert nb_ext, 'No notebook extension found for: %s' % nb
+			mw_ext = MainWindowExtension(self, obj, nb_ext)
+			self.extensions.add(mw_ext)
+		else:
+			PluginClass.extend(self, obj)
 
 
-ui_actions = (
-	# name, stock id, label, accelerator, tooltip, readonly
-	('save_version', 'gtk-save-as', _('S_ave Version...'), '<ctrl><shift>S', '', False), # T: menu item
-	('show_versions', None, _('_Versions...'), '', '', True), # T: menu item
-)
+@extends('Notebook')
+class NotebookExtension(ObjectExtension):
+
+	def __init__(self, plugin, notebook):
+		ObjectExtension.__init__(self, plugin, notebook)
+		self.plugin = plugin
+		self.notebook = notebook
+		self.detect_vcs()
+
+	def _get_notebook_dir(self):
+		if self.notebook.dir:
+			return self.notebook.dir
+		elif self.notebook.file:
+			return self.notebook.file.dir
+		else:
+			assert False, 'Notebook is not based on a file or folder'
+
+	def detect_vcs(self):
+		dir = self._get_notebook_dir()
+		self.vcs = VCS.detect_in_folder(dir)
+		if self.vcs:
+			# HACK - FIXME use proper FS signals here
+			# git requires changes to be added to staging, bzr does not
+			# so add a hook for when page is written, to update staging.
+			#
+			# For a more generic behavior, the update_staging is implemented
+			# for all version control systems. If not required - eg. bzr, hg,
+			# then nothing is done
+			self.notebook.connect_after('stored-page', lambda o, n: self.vcs.update_staging() )
+
+	def init_vcs(self, vcs):
+		dir = self._get_notebook_dir()
+		self.vcs = VCS.create(vcs, dir, dir)
+
+		if self.vcs:
+			with self.notebook.lock:
+				self.vcs.init()
+
+	def teardown(self):
+		if self.vcs:
+			self.vcs.disconnect_all()
 
 
-def async_commit_with_error(ui, vcs, msg, skip_no_changes=False):
-	'''Convenience method to wrap vcs.commit_async'''
-	def callback(ok, error, exc_info, data):
-		if error:
-			if isinstance(error, NoChangesError) and skip_no_changes:
-				logger.debug('No autosave version needed - no changes')
-			else:
-				logger.error('Error during async commit', exc_info=exc_info)
-				ErrorDialog(ui, error, exc_info).run()
-	vcs.commit_async(msg, callback=callback)
+def _monitor_thread(thread):
+	if thread.done:
+		if thread.error:
+			error = thread.exc_info[1]
+			logger.error('Error during async commit', exc_info=thread.exc_info)
+			ErrorDialog(None, error, thread.exc_info).run() # XXX None should be window
+		return False # stop signal
+	else:
+		return True # keep handler
+
+def monitor_thread(thread):
+	gobject.idle_add(_monitor_thread, thread)
+
+
+@extends('MainWindow')
+class MainWindowExtension(WindowExtension):
+
+	uimanager_xml = '''
+	<ui>
+	<menubar name='menubar'>
+		<menu action='file_menu'>
+			<placeholder name='versioning_actions'>
+				<menuitem action='save_version'/>
+				<menuitem action='show_versions'/>
+			</placeholder>
+		</menu>
+	</menubar>
+	</ui>
+	'''
+
+	def __init__(self, plugin, window, notebook_ext):
+		WindowExtension.__init__(self, plugin, window)
+		self.notebook_ext = notebook_ext
+		self._autosave_thread = None
+
+		if self.notebook_ext.vcs is None:
+			gaction = self.actiongroup.get_action('show_versions')
+			gaction.set_sensitive(False)
+		else:
+			if self.plugin.preferences['autosave']:
+				self.do_save_version_async()
+
+		def on_quit(o):
+			if self._autosave_thread and not self._autosave_thread.done:
+				self._autosave_thread.join()
+
+			if self.plugin.preferences['autosave']:
+				self.do_save_version()
+
+		self.window.ui.connect('quit', on_quit) # XXX
+
+	def do_save_version_async(self, msg=None):
+		if not self.notebook_ext.vcs:
+			return
+
+		if self._autosave_thread and not self._autosave_thread.done:
+			self._autosave_thread.join()
+
+		self._autosave_thread = FunctionThread(self.do_save_version, (msg,))
+		self._autosave_thread.start()
+		monitor_thread(self._autosave_thread)
+
+	def do_save_version(self, msg=None):
+		if not self.notebook_ext.vcs:
+			return
+
+		if not msg:
+			msg = _('Automatically saved version from zim')
+				# T: default version comment for auto-saved versions
+
+		self.window.ui.assert_save_page_if_modified() # XXX
+		try:
+			self.notebook_ext.vcs.commit(msg)
+		except NoChangesError:
+			logger.debug('No autosave version needed - no changes')
+
+	@action(_('S_ave Version...'), 'gtk-save-as', '<ctrl><shift>S', readonly=False) # T: menu item
+	def save_version(self):
+		self.window.ui.assert_save_page_if_modified() # XXX
+
+		if not self.notebook_ext.vcs:
+			vcs = VersionControlInitDialog().run()
+			if vcs is None:
+				return # Canceled
+
+			self.notebook_ext.init_vcs(vcs)
+			if self.notebook_ext.vcs:
+				gaction = self.actiongroup.get_action('show_versions')
+				gaction.set_sensitive(True)
+
+		with self.notebook_ext.notebook.lock:
+			SaveVersionDialog(self.window, self, self.notebook_ext.vcs).run()
+
+	@action(_('_Versions...')) # T: menu item
+	def show_versions(self):
+		self.window.ui.assert_save_page_if_modified() # XXX
+
+		dialog = VersionsDialog.unique(self, self.window,
+			self.notebook_ext.vcs,
+			self.notebook_ext.notebook,
+			self.window.ui.page # XXX
+		)
+		dialog.present()
 
 
 class NoChangesError(Error):
@@ -79,6 +236,7 @@ class NoChangesError(Error):
 		self.msg = _('No changes since last version')
 		# T: Short error descriotion
 
+
 class VCS(object):
 	"""
 	This class is the main entry for all Version Control System Stuff.
@@ -89,7 +247,6 @@ class VCS(object):
 	- create there a class inheriting from VCSApplicationBase \
 	- add here the stuff to manage it
 	"""
-	## TODO merge with VCSBackend class ?
 
 	# Enumeration of all available backends
 	BZR = _('Bazaar') # T: option value
@@ -100,17 +257,17 @@ class VCS(object):
 	def detect_in_folder(klass, dir):
 		"""Detect if a version control system has already been setup in the folder.
 		It also create the instance by calling the VCS.create() method
-		@param dir: a L{File} instance representing the notebook root folder
+		@param dir: a L{Dir} instance representing the notebook root folder
 		@returns: a L{VCSBackend} instance which will manage the versioning or C{None}
 		"""
 		name, root = klass._detect_in_folder(dir)
 
 		if name == 'bzr':
-			vcs = VCS.create(VCS.BZR, root)
+			vcs = VCS.create(VCS.BZR, root, dir)
 		elif name == 'hg':
-			vcs = VCS.create(VCS.HG, root)
+			vcs = VCS.create(VCS.HG, root, dir)
 		elif name == 'git':
-			vcs = VCS.create(VCS.GIT, root)
+			vcs = VCS.create(VCS.GIT, root, dir)
 		else:
 			# else maybe detected something, but no backend available
 			vcs = None
@@ -133,7 +290,7 @@ class VCS(object):
 				return 'bzr', path
 			elif path.subdir('.hg').exists():
 				return 'hg', path
-			elif path.subdir('.git').exists():
+			elif path.subdir('.git').exists() or path.file('.git').exists():
 				return 'git', path
 			elif path.subdir('.svn').exists():
 				return 'svn', path
@@ -170,18 +327,20 @@ class VCS(object):
 
 
 	@classmethod
-	def create(klass, vcs, dir):
+	def create(klass, vcs, vcs_dir, notebook_dir):
 		"""Build the required instance of a Version Control System
 
 		@param vcs: Version Control System to build (choose between VCS.BZR, VCS.HG, VCS.GIT)
-		@param dir: a L{File} instance representing the notebook root folder
+		@param vcs_dir: a L{Dir} instance representing the VCS root folder
+		@param notebook_dir: a L{Dir} instance representing the notebook root folder
+		(must be equal to or below vcs_dir)
 		@returns: a C{VCSBackend} instance setup with the required backend
 		"""
-		new_vcs = None
-		vcs_backend_klass = VCS.get_backend(vcs)
-		new_vcs = VCSBackend(dir, vcs_backend_klass(dir))
+		if not (notebook_dir == vcs_dir or notebook_dir.ischild(vcs_dir)):
+			raise AssertionError, 'Notebook %s is not part of version control dir %s' % (notebook_dir, vcs_dir)
 
-		return new_vcs
+		vcs_backend_klass = VCS.get_backend(vcs)
+		return VCSBackend(vcs_dir, vcs_backend_klass(vcs_dir, notebook_dir))
 
 	@classmethod
 	def check_dependencies(klass, vcs):
@@ -193,11 +352,10 @@ class VCS(object):
 
 
 
-class VCSBackend(object):
+class VCSBackend(ConnectorMixin):
 	"""Parent class for all VCS backend implementations.
 	It implements the required API.
 	"""
-	## TODO merge with VCS class ?
 
 	def __init__(self, dir, vcs_specific_app):
 		"""Initialize the instance in normal or test mode
@@ -215,11 +373,11 @@ class VCSBackend(object):
 		if not TEST_MODE:
 			# Avoid touching the bazaar repository with zim sources
 			# when we write to tests/tmp etc.
-			FS.connect('path-created', self.on_path_created)
-			FS.connect('path-moved', self.on_path_moved)
-			FS.connect('path-deleted', self.on_path_deleted)
-
-	# TODO: disconnect method - callbacks keep object alive even when plugin is disabled !
+			self.connectto_all(FS, (
+				'path-created',
+				'path-moved',
+				'path-deleted'
+			) )
 
 	@property
 	def vcs(self):
@@ -267,9 +425,7 @@ class VCSBackend(object):
 		@returns: nothing
 		"""
 		if path.ischild(self.root) and not self._ignored(path):
-			def wrapper():
-				self.vcs.add(path)
-			AsyncOperation(wrapper, lock=self.lock).start()
+			FunctionThread(self.vcs.add, (path,), lock=self._lock).start()
 
 
 	def on_path_moved(self, fs, oldpath, newpath):
@@ -282,13 +438,11 @@ class VCSBackend(object):
 		@returns: nothing
 		"""
 		if newpath.ischild(self.root) and not self._ignored(newpath):
-			def wrapper():
-				if oldpath.ischild(self.root):
-					# Parent of newpath needs to be versioned in order to make mv succeed
-					self.vcs.move(oldpath, newpath)
-				else:
-					self.vcs.add(newpath)
-			AsyncOperation(wrapper, lock=self.lock).start()
+			if oldpath.ischild(self.root):
+				# Parent of newpath needs to be versioned in order to make mv succeed
+				FunctionThread(self.vcs.move, (oldpath, newpath), lock=self._lock).start()
+			else:
+				FunctionThread(self.vcs.add, (newpath,), lock=self._lock).start()
 		elif oldpath.ischild(self.root) and not self._ignored(oldpath):
 			self.on_path_deleted(self, fs, oldpath)
 
@@ -299,10 +453,7 @@ class VCSBackend(object):
 		@param path: the L{UnixFile} object representing the path of the file or folder to delete
 		@returns: nothing
 		"""
-		def wrapper():
-			self.vcs.remove(path)
-		AsyncOperation(wrapper, lock=self.lock).start()
-
+		FunctionThread(self.vcs.remove, (path,), lock=self._lock).start()
 
 	@property
 	def modified(self):
@@ -311,14 +462,14 @@ class VCSBackend(object):
 		with self.lock:
 			return self.vcs.is_modified()
 
-	def get_status(self):
+	def get_status(self, **kwarg):
 		"""Returns repo status as a list of text lines
 
 		@returns: list of text lines (like a shell command result)
 		"""
 		status = list()
 		with self.lock:
-			status = self.vcs.status()
+			status = self.vcs.status(**kwarg)
 		return status
 
 	def get_diff(self, versions=None, file=None):
@@ -359,14 +510,6 @@ class VCSBackend(object):
 			self.vcs.add()
 			self.vcs.commit(None, msg)
 
-	def commit_async(self, msg, callback=None, data=None):
-		# TODO in generic baseclass have this default to using
-		# commit() + the wrapper call the callback
-		#~ print '!! ASYNC COMMIT'
-		operation = AsyncOperation(self._commit, (msg,),
-			lock=self._lock, callback=callback, data=data)
-		operation.start()
-
 	def revert(self, version=None, file=None):
 		with self.lock:
 			self.vcs.revert(file, version)
@@ -405,12 +548,17 @@ class VCSApplicationBase(object):
 	a folder.
 	"""
 
-	def __init__(self, root):
+	def __init__(self, vcs_dir, notebook_dir):
 		"""Constructor.
-		@param root: a L{Dir} instance representing the notebook root folder
+		@param vcs_dir: a L{Dir} instance representing the VCS root folder
+		@param notebook_dir: a L{Dir} instance representing the notebook root folder
+		(must be equal to or below vcs_dir)
 		"""
+		if not (notebook_dir == vcs_dir or notebook_dir.ischild(vcs_dir)):
+			raise AssertionError, 'Notebook %s is not part of version control dir %s' % (notebook_dir, vcs_dir)
 		self._app = self.build_bin_application_instance()
-		self.root = root
+		self.root = vcs_dir
+		self.notebook_dir = notebook_dir
 
 	@classmethod
 	def build_bin_application_instance(cls):
@@ -673,116 +821,16 @@ class VCSApplicationBase(object):
 		raise NotImplementedError
 
 
-class VersionControlPlugin(PluginClass):
-
-	plugin_info = {
-		'name': _('Version Control'), # T: plugin name
-		'description': _('''\
-This plugin adds version control for notebooks.
-
-This plugin supports the Bazaar, Git and Mercurial version control systems.
-
-This is a core plugin shipping with zim.
-'''), # T: plugin description
-		'author': 'Jaap Karssenberg & John Drinkwater & Damien Accorsi',
-		'help': 'Plugins:Version Control',
-	}
-
-	plugin_preferences = (
-		('autosave', 'bool', _('Autosave version on regular intervals'), False), # T: Label for plugin preference
-	)
-
-	def __init__(self, ui):
-		PluginClass.__init__(self, ui)
-		self.vcs = None
-		if self.ui.ui_type == 'gtk':
-			self.ui.add_actions(ui_actions, self)
-			self.ui.add_ui(ui_xml, self)
-			self.actiongroup.get_action('show_versions').set_sensitive(False)
-			if self.ui.notebook:
-				self.detect_vcs()
-			else:
-				self.ui.connect_after('open-notebook',
-					lambda o, n: self.detect_vcs() )
-
-			def on_quit(o):
-				if self.preferences['autosave']:
-					self.autosave()
-			self.ui.connect('quit', on_quit)
-
-	@classmethod
-	def check_dependencies(klass):
-		has_bzr = VCS.check_dependencies(VCS.BZR)
-		has_git  = VCS.check_dependencies(VCS.GIT)
-		has_hg  = VCS.check_dependencies(VCS.HG)
-		#TODO parameterize the return, so that a new backend will be automatically available
-		return has_bzr|has_hg|has_git, [('bzr', has_bzr, False), ('hg', has_hg, False), ('git', has_git, False)]
-
-	def detect_vcs(self):
-		dir = self._get_notebook_dir()
-		self.vcs = VCS.detect_in_folder(dir)
-		if self.vcs:
-			# HACK - FIXME use proper FS signals here
-			# git requires changes to be added to staging, bzr does not
-			# so add a hook for when page is written, to update staging.
-			#
-			# For a more generic behavior, the update_staging is implemented
-			# for all version control systems. If not required - eg. bzr, hg,
-			# then nothing is done
-			self.ui.notebook.connect_after('stored-page', lambda o, n: self.vcs.update_staging() )
-
-			self.actiongroup.get_action('show_versions').set_sensitive(True)
-			if self.preferences['autosave']:
-				self.autosave()
-
-	def _get_notebook_dir(self):
-		notebook  = self.ui.notebook
-		if notebook.dir:
-			return notebook.dir
-		elif notebook.file:
-			return notebook.file.dir
-		else:
-			assert 'Notebook is not based on a file or folder'
-
-	def autosave(self):
-		if not self.vcs:
-			return
-
-		if self.ui.page and self.ui.page.modified:
-			self.ui.save_page()
-
-		logger.info('Automatically saving version')
-		with self.ui.notebook.lock:
-			async_commit_with_error(self.ui, self.vcs,
-				_('Automatically saved version from zim'),
-				skip_no_changes=True )
-				# T: default version comment for auto-saved versions
-
-	def save_version(self):
-		if not self.vcs:
-			vcs = VersionControlInitDialog().run()
-			if vcs is None:
-				return # Cancelled
-			self.init_vcs(vcs)
-
-		if self.ui.page.modified:
-			self.ui.save_page()
-
-		with self.ui.notebook.lock:
-			SaveVersionDialog(self.ui, self.vcs).run()
-
-	def init_vcs(self, vcs):
-		dir = self._get_notebook_dir()
-		self.vcs = VCS.create(vcs, dir)
-
-		if self.vcs:
-			with self.ui.notebook.lock:
-				self.vcs.init()
-			self.actiongroup.get_action('show_versions').set_sensitive(True)
-
-	def show_versions(self):
-		dialog = VersionsDialog.unique(self, self.ui, self.vcs)
-		dialog.present()
+def get_side_by_side_app():
+	for dir in data_dirs('helpers/compare_files/'):
+		for name in dir.list(): # XXX need object list
+			file = dir.file(name)
+			if name.endswith('.desktop') and file.exists():
+				app = DesktopEntryFile(file)
+				if app.tryexec():
+					return app
+	else:
+		return None
 
 
 class VersionControlInitDialog(QuestionDialog):
@@ -816,9 +864,10 @@ class VersionControlInitDialog(QuestionDialog):
 
 class SaveVersionDialog(Dialog):
 
-	def __init__(self, ui, vcs):
+	def __init__(self, ui, window_ext, vcs):
 		Dialog.__init__(self, ui, _('Save Version'), # T: dialog title
 			button=(None, 'gtk-save'), help='Plugins:Version Control')
+		self.window_ext = window_ext
 		self.vcs = vcs
 
 		self.vbox.pack_start(
@@ -845,14 +894,13 @@ class SaveVersionDialog(Dialog):
 		window, textview = ScrolledTextView(text=''.join(status), monospace=True)
 		vbox.add(window)
 
-
 	def do_response_ok(self):
 		# notebook.lock already set by plugin.save_version()
 		buffer = self.textview.get_buffer()
 		start, end = buffer.get_bounds()
 		msg = buffer.get_text(start, end, False).strip()
 		if msg:
-			async_commit_with_error(self.ui, self.vcs, msg)
+			self.window_ext.do_save_version_async(msg)
 			return True
 		else:
 			return False
@@ -860,12 +908,12 @@ class SaveVersionDialog(Dialog):
 
 class VersionsDialog(Dialog):
 
-	# TODO put state in uistate ..
-
-	def __init__(self, ui, vcs):
+	def __init__(self, ui, vcs, notebook, page=None):
 		Dialog.__init__(self, ui, _('Versions'), # T: dialog title
 			buttons=gtk.BUTTONS_CLOSE, help='Plugins:Version Control')
+		self.notebook = notebook
 		self.vcs = vcs
+		self._side_by_side_app = get_side_by_side_app()
 
 		self.uistate.setdefault('windowsize', (600, 500), check=value_is_coord)
 		self.uistate.setdefault('vpanepos', 300)
@@ -894,8 +942,9 @@ class VersionsDialog(Dialog):
 		hbox = gtk.HBox(spacing=5)
 		vbox.pack_start(hbox, False)
 		hbox.pack_start(self.page_radio, False)
-		self.page_entry = PageEntry(self.ui.notebook)
-		self.page_entry.set_path(ui.page)
+		self.page_entry = PageEntry(self.notebook)
+		if page:
+			self.page_entry.set_path(page)
 		hbox.pack_start(self.page_entry, False)
 
 		# View annotated button
@@ -918,6 +967,13 @@ state. Or select multiple versions to see changes between those versions.
 		scrolled = ScrolledWindow(self.versionlist)
 		vbox.add(scrolled)
 
+		col = self.uistate.setdefault('sortcol', self.versionlist.REV_SORT_COL)
+		order = self.uistate.setdefault('sortorder', gtk.SORT_DESCENDING)
+		try:
+			self.versionlist.get_model().set_sort_column_id(col, order)
+		except:
+			logger.exception('Invalid sort column: %s %s', col, order)
+
 		# -----
 		vbox = gtk.VBox(spacing=5)
 		self.vpaned.pack2(vbox, resize=False)
@@ -981,11 +1037,11 @@ state. Or select multiple versions to see changes between those versions.
 			elif len(rows) == 1:
 				revert_button.set_sensitive(usepage)
 				diff_button.set_sensitive(True)
-				comp_button.set_sensitive(usepage)
+				comp_button.set_sensitive(bool(usepage and self._side_by_side_app))
 			else:
 				revert_button.set_sensitive(False)
 				diff_button.set_sensitive(True)
-				comp_button.set_sensitive(usepage)
+				comp_button.set_sensitive(bool(usepage and self._side_by_side_app))
 
 		def on_page_change(o):
 			pagesource = self._get_file()
@@ -1010,18 +1066,17 @@ state. Or select multiple versions to see changes between those versions.
 	def save_uistate(self):
 		self.uistate['vpanepos'] = self.vpaned.get_position()
 
+		col, order = self.versionlist.get_model().get_sort_column_id()
+		self.uistate['sortcol'] = col
+		self.uistate['sortorder'] = order
+
 	def _get_file(self):
 		if self.notebook_radio.get_active():
-			if self.ui.page.modified:
-				self.ui.save_page()
-
 			return None
 		else:
 			path = self.page_entry.get_path()
 			if path:
-				page = self.ui.notebook.get_page(path)
-				if page == self.ui.page and page.modified:
-					self.ui.save_page()
+				page = self.notebook.get_page(path)
 			else:
 				return None # TODO error message valid page name?
 
@@ -1055,7 +1110,7 @@ state. Or select multiple versions to see changes between those versions.
 			  # T: Detailed question, "%(page)s" is replaced by the page, "%(version)s" by the version id
 		) ).run():
 			self.vcs.revert(file=file, version=version)
-			self.ui.reload_page()
+			self.ui.reload_page() # XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
 			# TODO trigger vcs autosave here?
 
 	def show_changes(self):
@@ -1067,7 +1122,26 @@ state. Or select multiple versions to see changes between those versions.
 			# T: dialog title
 
 	def show_side_by_side(self):
-		print 'TODO - need config for an application like meld'
+		file = self._get_file()
+		versions = self.versionlist.get_versions()
+		if not (file and versions):
+			raise AssertionError
+
+		files = map(lambda v: self._get_tmp_file(file, v), versions)
+		if len(files) == 1:
+			tmp = TmpFile(file.basename + '--CURRENT', persistent=True)
+				# need to be persistent, else it is cleaned up before application spawned
+			tmp.writelines(file.readlines())
+			files.insert(0, tmp)
+
+		self._side_by_side_app.spawn(files)
+
+	def _get_tmp_file(self, file, version):
+		text = self.vcs.get_version(file, version)
+		tmp = TmpFile(file.basename + '--REV%s' % version, persistent=True)
+			# need to be persistent, else it is cleaned up before application spawned
+		tmp.writelines(text)
+		return tmp
 
 
 class TextDialog(Dialog):
@@ -1075,6 +1149,7 @@ class TextDialog(Dialog):
 	def __init__(self, ui, title, lines):
 		Dialog.__init__(self, ui, title, buttons=gtk.BUTTONS_CLOSE)
 		self.set_default_size(600, 300)
+		self.uistate.setdefault('windowsize', (600, 500), check=value_is_coord)
 		window, textview = ScrolledTextView(''.join(lines), monospace=True)
 		self.vbox.add(window)
 
